-----------------------------------------------------------------------------
-- Nom fichier      :  outils_adadoc.ads                                   --
-- Auteur           :  <a>Julien Burdy, Vincent Decorges EIA</a>           --
-- Date             :  <d>05.07.2001</d>                                   --
--                                                                         --
-- But              :  <p>Met  disposition les outils et type ncessaire  --
--                        pour le programme principale d'adadoc.</p>       --
--                                                                         --
-- Date de modif.   :                                                      --
-- Raison           :                                                      --
-- Modules appeles  :                                                      --
-- Mat. particulier :                                                      --
--                                                                         --
-- Compilation      : Gnat 3.13p                                           --
-- Edition de liens :                                                      --
-- Mode d'execution :                                                      --
-----------------------------------------------------------------------------

--<?>String non contrainte</?>
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Text_IO; use Ada.Text_IO;

package Outils_Adadoc is

--======================================================================================

   -------------------------------------------------------------------------------------
   --<?>Pattern d'ouverture et de fermeture de commentaire</?> =========================
   -------------------------------------------------------------------------------------
   Comm_Ouverture : constant String := "<?>";
   Comm_Fermeture : constant String := "</?>";
   -------------------------------------------------------------------------------------
   --<?>Pattern d'ouverture et de fermeture pour l'auteur</?>  =========================
   -------------------------------------------------------------------------------------
   Auteur_Ouverture : constant String := "<a>";
   Auteur_Fermeture : constant String := "</a>";
   -------------------------------------------------------------------------------------
   --<?>Pattern d'ouverture et de fermeture de date</?>        =========================
   -------------------------------------------------------------------------------------
   Date_Ouverture : constant String := "<d>";
   Date_Fermeture : constant String := "</d>";
   -------------------------------------------------------------------------------------
   --<?>Pattern d'ouverture et de fermeture de commentaire du paquetage</?>=============
   -------------------------------------------------------------------------------------
   Paquetage_Ouverture : constant String := "<p>";
   Paquetage_Fermeture : constant String := "</p>";

   -------------------------------------------------------------------------------------
   --   type T_Mot  ====================================================================
   -------------------------------------------------------------------------------------


   --<?>Nombre de caractre max d'un T_Mot</?>
   Max_Caractere : constant := 512;


   --<?>Pour le discrimiant</?>
   subtype T_Nbr_Caractere is Integer range 0 .. Max_Caractere;


   --<?>T_Mot pour le scanner</?>
   type T_Mot (Longueur : T_Nbr_Caractere := 0) is
      Record
         Contenu : String (1 .. Longueur);
      end record;


   --<?>un simple mot vide</?>
   Mot_Vide : constant T_Mot := (0, "");



   -------------------------------------------------------------------------------------
   --  Constructeur / Modificateurs sur le type T_Mot  =================================
   -------------------------------------------------------------------------------------

   ------------------------------------------------------------------------
   --    Nom   : Creer_Mot                                               --
   -- <?>But   : Permet de crer un mot du type T_Mot                    --
   --                                                                    --
   --       Out: T_Mot   : Le mot obtenu aprs cration                  --
   --    In    : Contenu : Ce que le mot doit contenir aprs cration</?>--
   ------------------------------------------------------------------------
   function Creer_Mot (Contenu : in String) return T_Mot;


   ------------------------------------------------------------------------
   --    Nom   : "&"                                                     --
   -- <?>But   : Permet d'appondre deux mots ensemble                    --
   --                                                                    --
   --    In    : Mot_1 : 1er mot                                         --
   --    In    : Mot_2 : 2me mot  appondre au 1er                      --
   --    Retour: T_Mot : Le mot obtenu</?>                               --
   ------------------------------------------------------------------------
   function "&" (Mot_1, Mot_2 : in T_Mot) return T_Mot;


   ------------------------------------------------------------------------
   --    Nom   : Appondre                                                --
   -- <?>But   : Permet d'appondre un caractre  un mot                 --
   --                                                                    --
   --    In Out: Mot : Mot dont on appond le caractre                   --
   --    In    : Car : caractre  appondre</?>                          --
   ------------------------------------------------------------------------
   procedure Appondre (Mot : in out T_Mot; Car : in Character);


   ------------------------------------------------------------------------
   --    Nom   : Scanner                                                 --
   -- <?>But   : Permet de "hachache" un texte source Ada 95 en mots     --
   --                                                                    --
   --       Out: T_Mot   : Le mot obtenu aprs hachage                   --
   --    In    : Fichier : Le fichier contenant le code source ADA 95</?>--
   ------------------------------------------------------------------------
   procedure Scanner (Mot : out T_Mot; Fichier : in Ada.Text_IO.File_Type);

   ------------------------------------------------------------------------
   --    Nom   : Scanner_Comment                                         --
   -- <?>But   : Permet de "hachache" un commentaireAda 95 en mots       --
   --                                                                    --
   --       Out: Commentaire   : Le commentaire                          --
   --    In    : Tag_Fermeture : Le tag de fermeture du type de          --
   --                         commentaire trait                         --
   --    In    : Fichier : Le fichier contenant le code source ADA 95</?>--
   ------------------------------------------------------------------------
   procedure Scanner_Comment (Commentaire   : out Unbounded_String ;
                              Tag_Fermeture : in String;
                              Fichier       : in Ada.Text_IO.File_Type);



   -------------------------------------------------------------------------------------
   --  Slecteurs sur le type T_Mot ====================================================
   -------------------------------------------------------------------------------------

   ------------------------------------------------------------------------
   --    Nom   : Contenu                                                 --
   -- <?>But   : Permet d'obtenir le contenu d'un mot                    --
   --                                                                    --
   --    In    : Mot    : Le mot                                         --
   --    Retour: String : Son contenu</?>                                --
   ------------------------------------------------------------------------
   function Contenu (Mot : in T_Mot) return String;


   ------------------------------------------------------------------------
   --    Nom   : Contenu_Mot_Clef                                        --
   -- <?>But   : Permet d'obtenir le contenu d'un mot en forant la      --
   --            juste syntaxe d'un mot clef Ada95 (en minuscule)        --
   --                                                                    --
   --    In    : Mot    : Le mot                                         --
   --    Retour: String : Son contenu (minusculis)</?>                  --
   ------------------------------------------------------------------------
   function Contenu_Mot_Clef (Mot : in T_Mot) return String;


   ------------------------------------------------------------------------
   --    Nom   : Est_Un_Mot_Clef                                         --
   -- <?>But   : Permet de savoir si un Mot est un mot clef Ada95        --
   --                                                                    --
   --    In    : Mot     : Le mot                                        --
   --    Retour: Boolean : True si c'est un mot clef</?>                 --
   ------------------------------------------------------------------------
   function Est_Un_Mot_Clef (Mot : in T_Mot) return Boolean;


   ------------------------------------------------------------------------
   --    Nom   : Longueur                                                --
   -- <?>But   : Permet d'obtenir la longueur d'un mot                   --
   --                                                                    --
   --    In    : Mot     : Le mot                                        --
   --    Retour: Natural : Sa longueur (entre 0 et Max_Caractere)</?>    --
   ------------------------------------------------------------------------
   function Longueur (Mot : in T_Mot) return Natural;



--======================================================================================
--======================================================================================

   -------------------------------------------------------------------------------------
   --  type T_PT_Objet  ================================================================
   -------------------------------------------------------------------------------------


   --<?>Genre d'objet  traiter</?>
   type T_Genre is (Variable, Constante, Procedur, Fonction, Type_Simple, Sous_Type,
                    Parametre, Paquetage, Clause, Clause_Use, Clause_Use_Type, Generique, Article,
                    Partie_Variante, Para_Poo, Except, Pragm, Fin, Comment_Gen, Comment_Auteur, Comment_Date);


   --<?>Type de paramtre pour les sous-programme et autre paramtre</?>
   type T_Para is (Entree, Sortie, Entree_Sortie, Retour);


   --<?>Pour la liste dynamique</?>
   type T_Objet;
   type T_PT_Objet is access T_Objet;


   --<?>Pour stocker les diffrents objets d'une spec</?>
   type T_Objet (Genre : T_Genre := Clause) is

      record

         --<?>Nom de l'objet</?>
         Nom           : T_Mot;
         Commentaire   : Unbounded_String;
         Objet_Suivant : T_PT_Objet;

         --Suivant le type de l'objet on a des champs differents
         case Genre is
            --<?>Les variables</?>
            when Variable =>
               T_Variable    : Unbounded_String;  --Type de la variable
               Init_Var      : Unbounded_String;  --Initialisation

            --<?>Les constantes</?>
            when Constante =>
               T_Constante   : Unbounded_String;  --Type de la constant
               Init_Const    : Unbounded_String;  --Initialisation

            --<?>Les paramtres de sous programme et autre</?>
            when Parametre =>
               --<?>Genre du parametre IN, OUT, etc...</?>
               Genre_Para    : T_Para;
               T_Parametre   : T_Mot;             --Type du paramtre
               Init_Para     : Unbounded_String;  --Initialisation

            --Les procdures
            when Procedur =>
               Para_Proc      : T_PT_Objet;        --Paramtres de la procedure
               Proc_Generique : Unbounded_String;  --Procdure gnerique
               Proc_Abstract  : Boolean := False;

            --Les fonctions
            when Fonction =>
               Para_Fonc      : T_PT_Objet;        --Paramtres de la fonction
               Fonc_Generique : Unbounded_String;  --Procdure gnerique
               Fonc_Abstract  : Boolean := False;

            --Type non article
            when Type_Simple =>
               Declaration    : Unbounded_String;  --Partie dclarative
               Type_Tagged    : T_PT_Objet;       --Type tagged

            --Sous-Type
            when Sous_Type =>
               Declaration_Sous_Type : Unbounded_String;  --Partie dclarative

            --Dclaration de paquetage
            when Paquetage =>
               --Pour l'instanciation d'un paquetage gnrique
               Instance_Generique : Unbounded_String;

            --Paramtre gnerique
            when Generique =>
               Parametre   : T_PT_Objet;

            --Partie Variante d'un article
            when Partie_Variante =>
               Nom_Discriminant : T_Mot;
               Champs_Variant   : T_PT_Objet;

            --Article qui peut tre  discriminant ou a partie variante
            when Article =>

               Discriminant    : T_PT_Objet;
               Champs          : T_PT_Objet;
               P_Variante      : T_PT_Objet;
               Article_Tagged  : T_PT_Objet; --On a un article tagged

            --Les options d'un type tagged pour la poo
            when Para_Poo =>
               Is_Limited        : Boolean := False;
               Is_Abstract       : Boolean := False;
               Inheritance       : T_Mot := Mot_Vide;

            --Pour les Pragma
            when Pragm =>
               Pragma_Init     : Unbounded_String;

            --Clause de contexte
            when Clause =>
               Instance      : Unbounded_String;

            --Clause use type
            when Clause_Use_Type =>
               null;

            --Clause use
            when Clause_Use =>
               null;

            --Les exceptions
            when Except =>
               null;

            --Les end
            when Fin =>
               null;

            --Ne fait pas partie du langage. Pour les tags de commentaire
            when Comment_Gen | Comment_Auteur | Comment_Date =>
               null;

         end case;
      end record;

   ---------------------------------------------------------------------------
   --  Modificateurs sur le type T_PT_Objet  =================================
   ---------------------------------------------------------------------------

   ------------------------------------------------------------------------
   --    Nom   : Inserer                                                 --
   -- <?>But   : Permet d'insrer un objet dans une table de Symbole     --
   --                                                                    --
   --    In Out: Tab_Symbole   : La table ou il faut insrer             --
   --    In    : Objet         : L'objet a insrer</?>                   --
   ------------------------------------------------------------------------
   procedure Inserer (Tab_Symbole : in out T_PT_Objet;
                      Objet       : in T_Objet);


   --------------------------------------------------------------------------
   --    Nom   : Inserer                                                   --
   -- <?>But   : Permet d'insrer un objet dans une table de Symbole       --
   --                                                                      --
   --    In Out: Tab_Symbole   : La table ou il faut insrer               --
   --    In    : Objet         : L'objet a insrer                         --
   --       Out: Pointeur      : Pointeur sur l'objet qui a t insrer</?>--
   --------------------------------------------------------------------------
   procedure Inserer (Tab_Symbole : in out T_PT_Objet;
                      Objet       : in T_Objet;
                      Pointeur    : out T_PT_Objet);


   ------------------------------------------------------------------------
   --    Nom   : Modifier_Parametre                                      --
   -- <?>But   : Permet de modifier en chane des objets de type         --
   --            paramtre                                               --
   --                                                                    --
   --    In Out: Sous_Programme: Le sous-programme ou il faut modifier   --
   --                         les paramtres                             --
   --    In    : Genre         : Genre du para (Entre,Sortie,...)       --
   --    In    : T_Para        : Le type qu'il faut affecter au paramtre--
   --    In    : Init          : La valeur par dfaut a affecter</?>     --
   ------------------------------------------------------------------------
   procedure Modifier_Parametre (Sous_Programme : in out T_PT_Objet;
                                 Genre          : in T_Para := Entree;
                                 T_Para         : in T_Mot;
                                 Init           : in Unbounded_String);



   ---------------------------------------------------------------------------
   --  Slecteurs sur le type T_PT_Objet  ====================================
   ---------------------------------------------------------------------------

   ------------------------------------------------------------------------
   --    Nom   : Pointer_Suivant                                         --
   -- <?>But   : Permet de rcuperer l'objet suivant dans la table du    --
   --            genre cherch                                           --
   --                                                                    --
   --    In Out: Table : Table contenant les objets                      --
   --    In    : Genre : Genre de l'objet recherch</?>                  --
   ------------------------------------------------------------------------
   procedure Pointer_Suivant (Table : in out T_PT_Objet; Genre : in T_Genre);


   ------------------------------------------------------------------------
   --    Nom   : Existe                                                  --
   -- <?>But   : Permet de savoir si un objet d'un certain genre existe  --
   --            dans la table.                                          --
   --                                                                    --
   --    In    : Table    : La table contenant les objets                --
   --    In    : Genre    : Le genre d'objet recherch                   --
   --    Retour: True si la table contien l'objet</?>                    --
   ------------------------------------------------------------------------
   function Existe (Table : in T_PT_Objet; Genre : in T_Genre) return Boolean;

   ------------------------------------------------------------------------
   --    Nom   : Para_To_String                                          --
   -- <?>But   : Renvoie une chane de caractre d'aprs un type de      --
   --            paramtres                                              --
   --                                                                    --
   --    In    : Table    : La table contenant les objets                --
   --    In    : Genre    : Le genre d'objet recherch                   --
   --    Retour: True si la table contien l'objet</?>                    --
   ------------------------------------------------------------------------
   function Para_To_String (Para : T_Para) return String;

end Outils_Adadoc;
